import QUnit from 'steal-qunit';
import { ViewModel } from './donejs-typeahead';

// ViewModel unit tests
QUnit.module('donejs-typeahead');

var exampleData = {
  my: 'my own value',
  myBoolean: true,
  myNumber: 1.23,
  notDefined: undefined
};

// QUnit.test('Default config values', function() {
//   var vm = new ViewModel();
// });


// vm
QUnit.test('Changing the default values', function() {
  var vm = new ViewModel({
    remoteHost: '127.0.0.1',
    displayValue: 'name'
  });
  QUnit.equal(vm.attr('remoteHost'), '127.0.0.1', 'remoteHost property changed');
  QUnit.equal(vm.attr('displayValue'), 'name', 'displayValue property changed');

});



// #restApiSearchResultToArray(data)
QUnit.test('data is not valid', function() {
  var vm = new ViewModel();
  QUnit.deepEqual(vm.restApiSearchResultToArray(undefined), [], 'Undefined');
  QUnit.deepEqual(vm.restApiSearchResultToArray(null), [], 'null');
  QUnit.deepEqual(vm.restApiSearchResultToArray(false), [], 'false');
  QUnit.deepEqual(vm.restApiSearchResultToArray(-1), [], '-1');
  QUnit.deepEqual(vm.restApiSearchResultToArray(), [], 'empty');
});

QUnit.test('data.data is not valid', function() {
  var vm = new ViewModel();
  QUnit.deepEqual(vm.restApiSearchResultToArray({ a: 1 }), [], 'Undefined');
  QUnit.deepEqual(vm.restApiSearchResultToArray({ data: {} }), [], 'empty object');
});

QUnit.test('the result obtained is not in the expected format ( {data: [object1, object2]} )', function() {
  var vm = new ViewModel();
  QUnit.deepEqual(vm.restApiSearchResultToArray({ a: 1 }), [], 'no data field');
  QUnit.deepEqual(vm.restApiSearchResultToArray({ data: 1 }), [], 'data is not an array');
  QUnit.deepEqual(vm.restApiSearchResultToArray({ data: 'hello' }), [], 'data is not an array');
  QUnit.deepEqual(vm.restApiSearchResultToArray([{ some: 'data' }]), [], 'not an object');
});



// #getSelectedElementByKey(key)
QUnit.test('Retrieve standard types', function() {
  var vm = new ViewModel();
  vm.attr('selectedElementObject', exampleData);
  QUnit.equal(vm.getSelectedElementByKey('my'), 'my own value', 'String value successfully retrieved');
  QUnit.equal(vm.getSelectedElementByKey('myBoolean'), true, 'Boolean value successfully retrieved');
  QUnit.equal(vm.getSelectedElementByKey('myNumber'), 1.23, 'Number value successfully retrieved');
});

QUnit.test('Try to retrieve non existing or invalid types', function() {
  var vm = new ViewModel();
  vm.attr('selectedElementObject', exampleData);
  QUnit.equal(vm.getSelectedElementByKey('undefinedKey'), null, 'Key does not exist');
  QUnit.equal(vm.getSelectedElementByKey('notDefined'), null, 'Key does not exist');

});
